using System;
using System.Collections.Generic;
using System.Text;

using System.Management.Automation;

namespace Microsoft.PowerShell.CmdletManagement
{
    [Cmdlet(VerbsCommon.Add, "PsSpecCmdletParameterSetEntry", SupportsShouldProcess = true)]
    public class AddPsSpecCmdletParameterEntryCommand : Cmdlet
    {
        [ValidateLength(2, 100)]
        [Parameter(Mandatory = true, ValueFromPipelineByPropertyName = true)]
        public string CmdletName
        {
            get { return cmdletName; }
            set { cmdletName = value; }
        }
        private string cmdletName = null;

        [ValidateLength(3, 50)]
        [Parameter(Mandatory = true, ValueFromPipelineByPropertyName = true)]
        public string ProjectName
        {
            get { return projectName; }
            set { projectName = value; }
        }
        private string projectName = null;

        [Parameter(Mandatory = true, ValueFromPipelineByPropertyName = true)]
        [ValidateLength(1, 100)]
        public String ParameterName
        {
            get
            {
                return _name;
            }
            set
            {
                _name = value;
            }
        }
        private string _name = "*";

        [Parameter(ValueFromPipelineByPropertyName = true)]
        public String ParameterSetName
        {
            get
            {
                return _parameterSetName;
            }
            set
            {
                if ((value != null) && (value.Length > 50))
                    throw new ArgumentOutOfRangeException("ParameterSetName");

                _parameterSetName = value;
            }
        }
        private string _parameterSetName = null;

        [Parameter(ValueFromPipelineByPropertyName = true)]
        public bool Mandatory
        {
            get { return mandatory; }
            set { mandatory = value; }
        }
        private bool mandatory = false;

        [Parameter(ValueFromPipelineByPropertyName = true)]
        [AllowNull]
        public int? Position
        {
            get { return position; }
            set { position = value; }
        }
        private int? position;

        [Parameter(ValueFromPipelineByPropertyName = true)]
        public bool ValueFromPipeline
        {
            get { return valueFromPipeline; }
            set { valueFromPipeline = value; }
        }
        private bool valueFromPipeline = false;

        [Parameter(ValueFromPipelineByPropertyName = true)]
        public bool ValueFromPipelineByPropertyName
        {
            get { return valueFromPipelineByPropertyName; }
            set { valueFromPipelineByPropertyName = value; }
        }
        private bool valueFromPipelineByPropertyName = false;

        [Parameter(ValueFromPipelineByPropertyName = true)]
        public bool ValueFromRemainingArguments
        {
            get { return valueFromRemainingArguments; }
            set { valueFromRemainingArguments = value; }
        }
        private bool valueFromRemainingArguments = false;

        [Parameter(ValueFromPipelineByPropertyName = true)]
        public string HelpMessage
        {
            get { return helpMessage; }
            set
            {
                if ((value != null) && (value.Length > 200))
                    throw new ArgumentOutOfRangeException("HelpMessage");

                helpMessage = value;
            }
        }
        private string helpMessage = null;

        [Parameter(ValueFromPipelineByPropertyName = true)]
        public string HelpMessageBaseName
        {
            get { return helpMessageBaseName; }
            set
            {
                if ((value != null) && (value.Length > 200))
                    throw new ArgumentOutOfRangeException("HelpMessageBaseName");

                helpMessageBaseName = value;
            }
        }
        private string helpMessageBaseName = null;

        [Parameter(ValueFromPipelineByPropertyName = true)]
        public string HelpMessageResourceId
        {
            get { return helpMessageResourceId; }
            set
            {
                if ((value != null) && (value.Length > 200))
                    throw new ArgumentOutOfRangeException("HelpMessageResourceId");

                helpMessageResourceId = value;
            }
        }
        private string helpMessageResourceId = null;

        [Parameter(Mandatory = false)]
        public String ServiceUri
        {
            get
            {
                return _serviceUri;
            }
            set
            {
                _serviceUri = value;
            }
        }
        private string _serviceUri = PsSpecCmdletUtils.Constants.ServiceUri;


        protected override void ProcessRecord()
        {
            if (ShouldProcess(
                String.Format("{0}", ServiceUri),
                String.Format(resources.CreatePsSpecCmdletParameterSetEntryShouldrocess, ParameterName)))
            {
                PsSpecCmdletService.CmdletService service =
                    new PsSpecCmdletService.CmdletService();
                service.Url = ServiceUri;
                
                service.UseDefaultCredentials = true;

                try
                {
                    service.NewParameterSetEntry(ProjectName, CmdletName, ParameterSetName, ParameterName,
                        Mandatory, Position, ValueFromPipeline, ValueFromPipelineByPropertyName, 
                        ValueFromRemainingArguments, HelpMessage, HelpMessageBaseName, HelpMessageResourceId);
                }
                catch (System.Web.Services.Protocols.SoapException e)
                {
                    WriteError(new ErrorRecord(e, "CMDLET_SERVICE_ERROR",
                                                ErrorCategory.InvalidOperation,
                                                String.Format(resources.CreatePsSpecCmdletParameterSetEntryShouldrocess, 
                                                    ParameterName)));
                }
            }
        }
    }
}
