using System;
using System.Collections.Generic;
using System.Text;

using System.Management.Automation;

namespace Microsoft.PowerShell.CmdletManagement
{
    [Cmdlet(VerbsCommon.New, "PsSpecCmdlet", SupportsShouldProcess = true)]
    public class NewPsSpecCmdletCommand : Cmdlet
    {
        [Parameter(Position = 1, ValueFromPipelineByPropertyName=true)]
        public String Snapin
        {
            get
            {
                return _snapin;
            }
            set
            {
                if ((value != null) && (value.Length > 200))
                    throw new ArgumentOutOfRangeException("Snapin");

                _snapin = value;
            }
        }
        private string _snapin;

        [Parameter(Mandatory = true, Position = 2, ValueFromPipelineByPropertyName = true)]
        [ValidateLength(1, 50)]
        public String Verb
        {
            get
            {
                return _verb;
            }
            set
            {
                _verb = value;
            }
        }
        private string _verb;

        [Parameter(Mandatory = true, Position = 3, ValueFromPipelineByPropertyName = true)]
        [ValidateLength(1, 50)]
        public String Noun
        {
            get
            {
                return _noun;
            }
            set
            {
                _noun = value;
            }
        }
        private string _noun;

        [Parameter(Mandatory = true, Position = 3, ValueFromPipelineByPropertyName = true)]
        [ValidateLength(1, 50)]
        public String ProjectName
        {
            get
            {
                return _project;
            }
            set
            {
                _project = value;
            }
        }
        private string _project;

        [Parameter(ValueFromPipelineByPropertyName = true)]
        public String TeamMembers
        {
            get
            {
                return _teamMembers;
            }
            set
            {
                if ((value != null) && (value.Length > 500))
                    throw new ArgumentOutOfRangeException("TeamMembers");

                _teamMembers = value;
            }
        }
        private string _teamMembers;

        [Parameter(ValueFromPipelineByPropertyName = true)]
        public String ShortDescription
        {
            get
            {
                return _shortDescription;
            }
            set
            {
                if ((value != null) && (value.Length > 2000))
                    throw new ArgumentOutOfRangeException("ShortDescription");

                _shortDescription = value;
            }
        }
        private string _shortDescription;

        [Parameter(ValueFromPipelineByPropertyName = true)]
        public String LongDescription
        {
            get
            {
                return _longDescription;
            }
            set
            {
                _longDescription = value;
            }
        }
        private string _longDescription;

        [Parameter(ValueFromPipelineByPropertyName = true)]
        public SwitchParameter SupportsShouldProcess
        {
            get
            {
                return _supportsShouldProcess;
            }
            set
            {
                _supportsShouldProcess = value;
            }
        }
        private SwitchParameter _supportsShouldProcess;

        [Parameter(ValueFromPipelineByPropertyName = true)]
        public String Notes
        {
            get
            {
                return _note;
            }
            set
            {
                _note = value;
            }
        }
        private string _note;

        [Parameter(ValueFromPipelineByPropertyName = true)]
        public String SpecExamples
        {
            get
            {
                return _specExamples;
            }
            set
            {
                _specExamples = value;
            }
        }
        private string _specExamples;

        [Parameter(ValueFromPipelineByPropertyName = true)]
        [AllowNull]
        [AllowEmptyCollection]
        public PsSpecHelpExample[] HelpExamples
        {
            get { return helpExamples; }
            set { helpExamples = value; }
        }
        private PsSpecHelpExample[] helpExamples = null;

        [Parameter(ValueFromPipelineByPropertyName = true)]
        public String OutputObject
        {
            get
            {
                return _outputObject;
            }
            set
            {
                if ((value != null) && (value.Length > 500))
                    throw new ArgumentOutOfRangeException("OutputObject");

                _outputObject = value;
            }
        }
        private string _outputObject;

        [Parameter(ValueFromPipelineByPropertyName = true)]
        public String OutputObjectDescription
        {
            get
            {
                return _outputObjectDescription;
            }
            set
            {
                _outputObjectDescription = value;
            }
        }
        private string _outputObjectDescription;

        [Parameter(ValueFromPipelineByPropertyName = true)]
        public String RelatedTo
        {
            get
            {
                return _relatedTo;
            }
            set
            {
                if ((value != null) && (value.Length > 1000))
                    throw new ArgumentOutOfRangeException("RelatedTo");

                _relatedTo = value;
            }
        }
        private string _relatedTo;

        [Parameter(ValueFromPipelineByPropertyName = true)]
        public String DefaultParameterSetName
        {
            get
            {
                return _defaultParameterSetName;
            }
            set
            {
                if ((value != null) && (value.Length > 50))
                    throw new ArgumentOutOfRangeException("DefaultParameterSetName");

                _defaultParameterSetName = value;
            }
        }
        private string _defaultParameterSetName;

        [Parameter(ValueFromPipelineByPropertyName = true)]
        public String Custom1
        {
            get
            {
                return _custom1;
            }
            set
            {
                _custom1 = value;
            }
        }
        private string _custom1;

        [Parameter(ValueFromPipelineByPropertyName = true)]
        public String Custom2
        {
            get
            {
                return _custom2;
            }
            set
            {
               _custom2 = value;
            }
        }
        private string _custom2;

        [Parameter(ValueFromPipelineByPropertyName = true)]
        public String Custom3
        {
            get
            {
                return _custom3;
            }
            set
            {
               _custom3 = value;
            }
        }
        private string _custom3;

        [Parameter(ValueFromPipelineByPropertyName = true)]
        public String Custom4
        {
            get
            {
                return _custom4;
            }
            set
            {
               _custom4 = value;
            }
        }
        private string _custom4;

        [Parameter(ValueFromPipelineByPropertyName = true)]
        public String Status
        {
            get
            {
                return _status;
            }
            set
            {
                if ((value != null) && (value.Length > 100))
                    throw new ArgumentOutOfRangeException("Status");

                _status = value;
            }
        }
        private string _status;

        [Parameter(ValueFromPipelineByPropertyName = true)]
        public String HelpStatus
        {
            get
            {
                return _helpStatus;
            }
            set
            {
                if ((value != null) && (value.Length > 100))
                    throw new ArgumentOutOfRangeException("HelpStatus");

                _helpStatus = value;
            }
        }
        private string _helpStatus;

        [Parameter(ValueFromPipelineByPropertyName = true)]
        public String HelpShortDescription
        {
            get
            {
                return _helpShortDescription;
            }
            set
            {
                if ((value != null) && (value.Length > 2000))
                    throw new ArgumentOutOfRangeException("HelpShortDescription");

                _helpShortDescription = value;
            }
        }
        private string _helpShortDescription;

        [Parameter(ValueFromPipelineByPropertyName = true)]
        public String HelpLongDescription
        {
            get
            {
                return _helpLongDescription;
            }
            set
            {
                _helpLongDescription = value;
            }
        }
        private string _helpLongDescription;

        [Parameter(ValueFromPipelineByPropertyName = true)]
        public String InputObject
        {
            get
            {
                return _helpInputObject;
            }
            set
            {
                if ((value != null) && (value.Length > 500))
                    throw new ArgumentOutOfRangeException("InputObject");

                _helpInputObject = value;
            }
        }
        private string _helpInputObject;

        [Parameter(ValueFromPipelineByPropertyName = true)]
        public String HelpInputObjectDescription
        {
            get
            {
                return _helpInputObjectDescription;
            }
            set
            {
                _helpInputObjectDescription = value;
            }
        }
        private string _helpInputObjectDescription;

        [Parameter(ValueFromPipelineByPropertyName = true)]
        public String HelpOutputObjectDescription
        {
            get
            {
                return _helpOutputObjectDescription;
            }
            set
            {
                _helpOutputObjectDescription = value;
            }
        }
        private string _helpOutputObjectDescription;

        [Parameter(ValueFromPipelineByPropertyName = true)]
        public String HelpAdditionalNotes
        {
            get
            {
                return _helpAdditionalNotes;
            }
            set
            {
                _helpAdditionalNotes = value;
            }
        }
        private string _helpAdditionalNotes;

        [Parameter(Mandatory = false)]
        public String ServiceUri
        {
            get
            {
                return _serviceUri;
            }
            set
            {
                _serviceUri = value;
            }
        }
        private string _serviceUri = PsSpecCmdletUtils.Constants.ServiceUri;

        protected override void ProcessRecord()
        {
            if (ShouldProcess(
                String.Format("{0}", ServiceUri),
                String.Format(resources.CreateCmdletShouldProcess, Verb, Noun)))
            {
                PsSpecCmdletService.CmdletService service =
                    new PsSpecCmdletService.CmdletService();
                service.Url = ServiceUri;
                
                service.UseDefaultCredentials = true;

                try
                {
                    PsSpecCmdletService.PsSpecHelpExample[] outgoingHelpExamples = 
                        new PsSpecCmdletService.PsSpecHelpExample[0];

                    // Populate the help examples
                    if (HelpExamples != null)
                    {
                        outgoingHelpExamples =
                            new PsSpecCmdletService.PsSpecHelpExample[HelpExamples.Length];

                        for (int counter = 0; counter < HelpExamples.Length; counter++)
                        {
                            PsSpecCmdletService.PsSpecHelpExample newExample =
                                new PsSpecCmdletService.PsSpecHelpExample();
                            PsSpecHelpExample incomingExample = HelpExamples[counter];

                            newExample.Command = incomingExample.Command;
                            newExample.Output = incomingExample.Output;
                            newExample.Description = incomingExample.Description;

                            outgoingHelpExamples[counter] = newExample;
                        }
                    }

                    // Create the cmdlet
                    service.NewCmdlet(Snapin, Verb, Noun, ProjectName, TeamMembers, ShortDescription,
                        HelpShortDescription, LongDescription, HelpLongDescription,
                        Status, HelpStatus, SupportsShouldProcess, Notes, SpecExamples, outgoingHelpExamples,
                        InputObject, HelpInputObjectDescription, OutputObject, OutputObjectDescription, 
                        HelpOutputObjectDescription, DefaultParameterSetName,
                        RelatedTo, Custom1, Custom2, Custom3, Custom4, HelpAdditionalNotes);

                    // Retrieve the cmdlet to output
                    GetPsSpecCmdletCommand psSpecCmdletCommand = new GetPsSpecCmdletCommand();
                    psSpecCmdletCommand.ProjectName = ProjectName;
                    psSpecCmdletCommand.Name = this.Verb + "-" + this.Noun;
                    psSpecCmdletCommand.ServiceUri = ServiceUri;

                    WriteObject(psSpecCmdletCommand.Invoke(), true);
                }
                catch (System.Web.Services.Protocols.SoapException e)
                {
                    WriteError(new ErrorRecord(e, "CMDLET_SERVICE_ERROR", 
                                                ErrorCategory.InvalidOperation,
                                                String.Format(resources.CreateCmdletShouldProcess, 
                                                    Verb, Noun)));
                }
            }
        }
    }
}
